/* winbond841.c: A Linux PCI network adapter skeleton device driver. */
/*
	Maintained by Geliang <lge@winbond.com.tw>

	Much code comes from Donald Becker's rl100a.c driver,
	versions 1.00.  
	
	
	-----<snip>----	
	
	Written 1998 by Donald Becker
	
	This software may be used and distributed according to the terms
	of the GNU Public License (GPL), incorporated herein by reference.
	Drivers based on this skeleton fall under the GPL and must retain
	the authorship (implicit copyright) notice.
	
	 The author may be reached as becker@scyld.com, or C/O Scyld
	Computing Corporation 410 Severn Ave., Suite 210 Annapolis
	MD 21403
	
	
	-----<snip>----
	
	This software may be used and distributed according to the terms
	of the GNU Public License, incorporated herein by reference.
	
	
	
	
*/

static const char *version =
"Winbond W89C841F 10/100M Ethernet Control version 1.00\n";


/* A few user-configurable values.   These may be modified when a driver
   module is loaded.*/

static int debug = 1;			/* 1 normal messages, 0 quiet .. 7 verbose. */
static int max_interrupt_work = 20;
static int min_pci_latency = 32;
static long pciaddr_dxx;		/*  base address of Dxx registers */

/* Maximum number of multicast addresses to filter (vs. Rx-all-multicast).
   The '841 uses a 64 element hash table based on the Ethernet CRC.  */
static int multicast_filter_limit = 32;

/* Set the copy breakpoint for the copy-only-tiny-frames scheme.
   Setting to > 1518 effectively disables this feature. */
static int rx_copybreak = 0;

/* Used to pass the media type, etc.
   Both 'options[]' and 'full_duplex[]' should exist for driver
   interoperability.
   The media type is usually passed in 'options[]'.
*/
#define MAX_UNITS 8		/* More are supported, limit only on options */
static int options[MAX_UNITS] = {-1, -1, -1, -1, -1, -1, -1, -1};
static int full_duplex[MAX_UNITS] = {-1, -1, -1, -1, -1, -1, -1, -1};

/* Operational parameters that are set at compile time. */

/* Keep the ring sizes a power of two for compile efficiency.
   The compiler will convert <unsigned>'%'<2^N> into a bit mask.
   Making the Tx ring too large decreases the effectiveness of channel
   bonding and packet priority.
   There are no ill effects from too-large receive rings. */
#define TX_RING_SIZE	16
#define RX_RING_SIZE	32

/* Operational parameters that usually are not changed. */
/* Time in jiffies before concluding the transmitter is hung. */
#define TX_TIMEOUT  (10*HZ)

#define PKT_BUF_SZ		1536			/* Size of each temporary Rx buffer.*/

/* Include files, designed to support most kernel versions 2.0.0 and later. */
#include <linux/config.h>
#include <linux/version.h>
#ifdef MODULE
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#else
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/malloc.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <asm/processor.h>		/* Processor type for cache alignment. */
#include <asm/bitops.h>
#include <asm/io.h>

/* This driver was written to use PCI memory space, however some x86 systems
   work only with I/O space accesses. */
#ifndef __alpha__
#define USE_IO
#endif

#ifdef USE_IO
#undef readb
#undef readw
#undef readl
#undef writeb
#undef writew
#undef writel
#define readb inb
#define readw inw
#define readl inl
#define writeb outb
#define writew outw
#define writel outl
#endif

/* Kernel compatibility defines, some common to David Hinds' PCMCIA package.
   This is only in the support-all-kernels source code. */

#define RUN_AT(x) (jiffies + (x))

#if (LINUX_VERSION_CODE >= 0x20100)
char kernel_version[] = UTS_RELEASE;
#else
#ifndef __alpha__
#define ioremap vremap
#define iounmap vfree
#endif
#endif
#if defined(MODULE) && LINUX_VERSION_CODE > 0x20115
MODULE_AUTHOR("Winbond Electronics");
MODULE_DESCRIPTION("Winbond W89C841F PCI Fast Ethernet driver");
MODULE_PARM(max_interrupt_work, "i");
MODULE_PARM(min_pci_latency, "i");
MODULE_PARM(debug, "i");
MODULE_PARM(rx_copybreak, "i");
MODULE_PARM(multicast_filter_limit, "i");
MODULE_PARM(options, "1-" __MODULE_STRING(MAX_UNITS) "i");
MODULE_PARM(full_duplex, "1-" __MODULE_STRING(MAX_UNITS) "i");
#endif
#if LINUX_VERSION_CODE < 0x20123
#define test_and_set_bit(val, addr) set_bit(val, addr)
#endif
#if LINUX_VERSION_CODE <= 0x20139
#define	net_device_stats enet_statistics
#else
#define NETSTATS_VER2
#endif
#if LINUX_VERSION_CODE < 0x20155  ||  defined(CARDBUS)
/* Grrrr, the PCI code changed, but did not consider CardBus... */
#include <linux/bios32.h>
#define PCI_SUPPORT_VER1
#else
#define PCI_SUPPORT_VER2
#endif
#if LINUX_VERSION_CODE < 0x20159
#define dev_free_skb(skb) dev_kfree_skb(skb, FREE_WRITE);
#else
#define dev_free_skb(skb) dev_kfree_skb(skb);
#endif


/*
				Theory of Operation

I. Board Compatibility

This driver is for the Winbond w89c841 chip.

II. Board-specific settings

None.

III. Driver operation

The most different between W840 and W841 is the IO space. W841 needs two I/O space while W840 
needs only one. 

IV. Notes

None.

IVc. Errata

Datasheet of W89C841F can be obtained from www.winbond.com.tw

*/



/*
  PCI probe table.
*/
static struct device *w841_probe1(int pci_bus, int devfn, struct device *dev, long ioaddr, int irq, int chp_idx, int fnd_cnt);
#ifdef USE_IO
#define W841_FLAGS (PCI_USES_IO | PCI_ADDR0 | PCI_USES_MASTER)
#else
#define W841_FLAGS (PCI_USES_MEM | PCI_ADDR2 | PCI_USES_MASTER)
#endif

enum pci_flags_bit {			/*F04 Command and Status Register */
	PCI_USES_IO=1,				/*IO Space */ 
	PCI_USES_MEM=2,             /*Memory Space */
	PCI_USES_MASTER=4,          /*Bus Master */
	PCI_ADDR0=0x10<<0, 
	PCI_ADDR1=0x10<<1, 
	PCI_ADDR2=0x10<<2, 
	PCI_ADDR3=0x10<<3,
};
struct pci_id_info {
	const char *name;
	u16	vendor_id, device_id, device_id_mask, flags;
	int io_size;				/* For Cxx Register */
	int io_size_dxx;			/* For Dxx Register */
	struct device *(*probe1)(int pci_bus, int pci_devfn, struct device *dev,
							 long ioaddr, int irq, int chip_idx, int fnd_cnt);
};

static struct pci_id_info pci_tbl[] = {
	{"Winbond W89c841", 0x1050, 0x0841, 0xffff, W841_FLAGS, 64, 256, w841_probe1},
	{0,},						/* 0 terminated list. */
};


/* A chip capabilities table, matching the entries in pci_tbl[] above. */
enum chip_capability_flags {CanHaveMII=1, HasBrokenTx=2};
struct chip_info {
	char *chip_name;
	int io_size;          /*W841 has two groups registers -- Cxx and Dxx, so two IO space are needed. */ 
	int io_size_dxx;
	int flags;
	void (*media_timer)(unsigned long data);
} static skel_netdrv_tbl[] = {
	{"Winbond W89c841 Fast Ethernet", 64, 256, CanHaveMII | HasBrokenTx, },
	                                  /*IO_size has been changed */
};

/* Offsets to  Registers. */
enum w841_offsets {
		
	/* Cxx Registers */
	PCIBusCfg=0x00,			/* PCI Bus Control Register */
	TxStartDemand=0x04,       		/* Transmit Start Demand Register */
	RxStartDemand=0x08,         		/* Receive Start Demand Register */  
	RxRingPtr=0x0C,			/* Receive Descriptor List Address */
	TxRingPtr=0x10,             		/* Transmit Descriptor List Address */
	IntrStatus=0x14,            		/* Interrupt Status Register */ 
	IntrEnable=0x18,            		/* Interrupt Mask Register */
	NetworkConfig=0x1C,         		/* Network Configurature Register */ 
	RxMissed=0x20,              		/* Frame Discarded Counter Register */
	CurTxDescAddr=0x24,         		/* Current Transmit Descriptor Address */
	CurTxBufAddr=0x28,          		/* Current Transmit Buffer Address */
	CurRxDescAddr=0x2C,		/* Current Receive Descriptor Address */       
	CurRxBufAddr=0x30,	        		/* Current Receive Buffer Address */
	MulticastFilter0=0x34,			/* Multicast Address0 */
	MulticastFilter1=0x38,			/* Multicast Address1 */
	GPTimer=0x3C,			/* General Timer Register */

	/* Dxx Registers */
	WakeUpControlStatus=0x00,
	AdvancedWakeUpFrameOffset0=0x04,
	AdvancedWakeUpFrameOffset1=0x08,
	WakeUpFrameB0B1CRC=0x0C,
	WakeUpFrameB2B3CRC=0x10,
	WakeUpFrameB4A0CRC=0x14,
	WakeUpFrameA1A2CRC=0x18,
	WakeUpFrameA3A4CRC=0x1C,
	BasicWakeUpFrame0ByteMask0=0x20,
	BasicWakeUpFrame0ByteMask1=0x24,
	BasicWakeUpFrame0ByteMask2=0x28,
	BasicWakeUpFrame0ByteMask3=0x2C,
	BasicWakeUpFrame1ByteMask0=0x30,
	BasicWakeUpFrame1ByteMask1=0x34,
	BasicWakeUpFrame1ByteMask2=0x38,
	BasicWakeUpFrame1ByteMask3=0x3C,
	BasicWakeUpFrame2ByteMask0=0x40,
	BasicWakeUpFrame2ByteMask1=0x44,
	BasicWakeUpFrame2ByteMask2=0x48,
	BasicWakeUpFrame2ByteMask3=0x4C,
	BasicWakeUpFrame3ByteMask0=0x50,
	BasicWakeUpFrame3ByteMask1=0x54,
	BasicWakeUpFrame3ByteMask2=0x58,
	BasicWakeUpFrame3ByteMask3=0x5C,
	BasicWakeUpFrame4ByteMask0=0x60,
	BasicWakeUpFrame4ByteMask1=0x64,
	BasicWakeUpFrame4ByteMask2=0x68,
	BasicWakeUpFrame4ByteMask3=0x6C,
    	AdvancedWakeUpFrame0ByteMask0=0x70,
	AdvancedWakeUpFrame0ByteMask1=0x74,
	AdvancedWakeUpFrame0ByteMask2=0x78,
	AdvancedWakeUpFrame0ByteMask3=0x7C,
	AdvancedWakeUpFrame1ByteMask0=0x80,
	AdvancedWakeUpFrame1ByteMask1=0x84,
	AdvancedWakeUpFrame1ByteMask2=0x88,
	AdvancedWakeUpFrame1ByteMask3=0x8C,
	AdvancedWakeUpFrame2ByteMask0=0x90,
	AdvancedWakeUpFrame2ByteMask1=0x94,
	AdvancedWakeUpFrame2ByteMask2=0x98,
	AdvancedWakeUpFrame2ByteMask3=0x9C,
	AdvancedWakeUpFrame3ByteMask0=0xA0,
	AdvancedWakeUpFrame3ByteMask1=0xA4,
	AdvancedWakeUpFrame3ByteMask2=0xA8,
	AdvancedWakeUpFrame3ByteMask3=0xAC,
	AdvancedWakeUpFrame4ByteMask0=0xB0,
	AdvancedWakeUpFrame4ByteMask1=0xB4,
	AdvancedWakeUpFrame4ByteMask2=0xB8,
	AdvancedWakeUpFrame4ByteMask3=0xBC,
	
	BootRom=0xC0,			/* BootRoom Access Register*/
	EECtrl=0xC4,           			/* EEPROM Access Register*/
	MIICtrl=0xC8,          			/* MII Control Access Register*/
	StationAddr=0xCC,      		/* Phisical Address 0 Register*/
	
	PhysicalAddress1=0xD0,
	ScanFIFOControl=0xD4,
	FIFOData=0xD8,
	FunctionEvent=0xF0,
	FunctionEventMask=0xF4,
	FunctionPresentStatus=0xF8,
	FunctionForceEvent=0xFC,
	
	
};

/* Bits in the interrupt status/enable registers. */
/* The bits in the Intr Status/Enable registers, mostly interrupt sources. */
enum intr_status_bits {
	NormalIntr=0x10000,			/*Normal Interrupt Report */
	AbnormalIntr=0x8000,			/*Abnormal Interrupt Report */
	IntrPCIErr=0x2000,			/*Bus Error */
	TimerInt=0x800,			/*Timer Expired */
	IntrRxDied=0x100,			/*Transmit Early */
	RxNoBuf=0x80,			/*Receive Buffer Unavailable */
	IntrRxDone=0x40,			/*Receive Complete */
	TxFIFOUnderflow=0x20,		/*Transmit FIFO Underflow */
	RxErrIntr=0x10,			/*Receive Error */
	TxIdle=0x04,			/*Buffer Unavailable */
	IntrTxStopped=0x02,			/*Transmit Process in idle state */
	IntrTxDone=0x01,			/*Transmit Complete */
};

/* Bits in the NetworkConfig register. */
enum rx_mode_bits {
	AcceptErr=0x80,			/*Accept Error Packet*/
	AcceptRunt=0x40,           		/*Accept Runt Packet */
	AcceptBroadcast=0x20,       		/*Accept Broadcast Packet*/
	AcceptMulticast=0x10,			/*Accept Multicast Packet*/
	AcceptAllPhys=0x08,			/*Accept All Physical Packet*/
	AcceptMyPhys=0x04,			/*Receive On*/
};


/* The Tulip Rx and Tx buffer descriptors. */
struct w841_rx_desc {
	s32 status;
	s32 length;
	u32 buffer1;
	u32 next_desc;
};

struct w841_tx_desc {
	s32 status;
	s32 length;
	u32 buffer1, buffer2;				/* We use only buffer 1.  */
};

/* Bits in network_desc.status */
enum desc_status_bits {
	DescOwn=0x80000000, 
	DescEndRing=0x02000000, 
	DescUseLink=0x01000000,
	DescWholePkt=0x60000000, 
	DescStartPkt=0x20000000, 
	DescEndPkt=0x40000000,
	DescIntr=0x80000000,
};

struct netdev_private {
	/* Descriptor rings first for alignment. */
	struct w841_rx_desc rx_ring[RX_RING_SIZE];
	struct w841_tx_desc tx_ring[TX_RING_SIZE];
	struct device *next_module;				/* Link for devices of this type. */
	const char *product_name;
	/* The addresses of receive-in-place skbuffs. */
	struct sk_buff* rx_skbuff[RX_RING_SIZE];
	/* The saved address of a sent-in-place packet/buffer, for later free(). */
	struct sk_buff* tx_skbuff[TX_RING_SIZE];
	struct net_device_stats stats;
	struct timer_list timer;	/* Media monitoring timer. */
	/* Frequently used values: keep some adjacent for cache effect. */
	int chip_id;
	int flags;
	unsigned char pci_bus, pci_devfn;
	int csr6;
	struct w841_rx_desc *rx_head_desc;
	unsigned int cur_rx, dirty_rx;		/* Producer/consumer ring indices */
	unsigned int cur_tx, dirty_tx;
	unsigned int rx_buf_sz;				/* Based on MTU+slack. */
	unsigned int tx_full:1;				/* The Tx queue is full. */
	/* These values are keep track of the transceiver/media in use. */
	unsigned int full_duplex:1;			/* Full-duplex operation requested. */
	unsigned int duplex_lock:1;
	unsigned int medialock:1;			/* Do not sense media. */
	unsigned int default_port:4;			/* Last dev->if_port value. */
	/* MII transceiver section. */
	int mii_cnt;					/* MII device addresses. */
	u16 advertising;				/* NWay media advertisement */
	unsigned char phys[2];				/* MII device addresses. */
	u32 pad[4];					/* Used for 32-byte alignment */
};

static int  eeprom_read(long ioaddr, int location);
static int  mdio_read(struct device *dev, int phy_id, int location);
static void mdio_write(struct device *dev, int phy_id, int location, int value);
static int  netdev_open(struct device *dev);
static void check_duplex(struct device *dev);
static void netdev_timer(unsigned long data);
static void tx_timeout(struct device *dev);
static void init_ring(struct device *dev);
static int  start_tx(struct sk_buff *skb, struct device *dev);
static void intr_handler(int irq, void *dev_instance, struct pt_regs *regs);
static void netdev_error(struct device *dev, int intr_status);
static int  netdev_rx(struct device *dev);
static void netdev_error(struct device *dev, int intr_status);
static inline unsigned ether_crc(int length, unsigned char *data);
static void set_rx_mode(struct device *dev);
static struct net_device_stats *get_stats(struct device *dev);
static int mii_ioctl(struct device *dev, struct ifreq *rq, int cmd);
static int  netdev_close(struct device *dev);



/* A list of our installed devices, for removing the driver module. */
static struct device *root_net_dev = NULL;

/* Ideally we would detect all network cards in slot order.  That would
   be best done a central PCI probe dispatch, which wouldn't work
   well when dynamically adding drivers.  So instead we detect just the
   cards we know about in slot order. */

static int pci_etherdev_probe(struct device *dev, struct pci_id_info pci_tbl[])
{
	int cards_found = 0;
	int pci_index = 0;
	int sign1,sign2;
	unsigned char pci_bus, pci_device_fn;

	if ( ! pcibios_present())
		return -ENODEV;

	for (;pci_index < 0xff; pci_index++) {
		u16 vendor, device, pci_command, new_command, signature1, signature2;
		int chip_idx, irq;
		long pciaddr;
		long ioaddr;		/* Base-Address for Cxx in iospace*/
				
		if (pcibios_find_class (PCI_CLASS_NETWORK_ETHERNET << 8, pci_index,
								&pci_bus, &pci_device_fn)
			!= PCIBIOS_SUCCESSFUL)
			break;
		pcibios_read_config_word(pci_bus, pci_device_fn,
								 PCI_VENDOR_ID, &vendor);
		pcibios_read_config_word(pci_bus, pci_device_fn,
								 PCI_DEVICE_ID, &device);
		pcibios_read_config_word(pci_bus, pci_device_fn,
								 0x40, &signature1);		
		pcibios_read_config_word(pci_bus, pci_device_fn,
								 0x40, &signature2);
		sign1= signature1 & 0xFF;
		sign2= signature2 & 0xFF;

		/* The signature register F40's will return 70H at 2Nth read and 44H at 2N+1th read.*/
		for (chip_idx = 0; pci_tbl[chip_idx].vendor_id; chip_idx++)
			if ( ( sign1==0x70 && sign2==0x44 ) || ( sign1==0x44 && sign2==0x70 ))
				break;
		if (pci_tbl[chip_idx].vendor_id == 0) 		/* Compiled out! */
			continue;

		{
#if defined(PCI_SUPPORT_VER2)
			struct pci_dev *pdev = pci_find_slot(pci_bus, pci_device_fn);
#ifdef USE_IO
			pciaddr = pdev->base_address[0] & ~3;	/* Read Cxx base address from configure register*/
			pciaddr_dxx= pdev->base_address[1]& ~3;	/* Read Dxx base address from configure register*/
			
#else
			pciaddr = pdev->base_address[2];
#endif
			irq = pdev->irq;
#else
			u32 pci_memaddr;
			u8 pci_irq_line;
			pcibios_read_config_byte(pci_bus, pci_device_fn,
									 PCI_INTERRUPT_LINE, &pci_irq_line);
#ifdef USE_IO
			pcibios_read_config_dword(pci_bus, pci_device_fn,
									  PCI_BASE_ADDRESS_0, &pci_memaddr);
			pciaddr = pci_memaddr & ~1;	/*Read from F10 and valuate to pciaddr */
			pcibios_read_config_dword(pci_bus, pci_device_fn,
									  PCI_BASE_ADDRESS_1, &pci_memaddr);
			pciaddr_dxx = pci_memaddr & ~1;

			
#else
			pcibios_read_config_dword(pci_bus, pci_device_fn,
									  PCI_BASE_ADDRESS_2, &pci_memaddr);
			pciaddr = pci_memaddr;
#endif
			irq = pci_irq_line;
#endif
		}

		if (debug > 2)
			printk(KERN_INFO "Found %s at PCI address %#lx, IRQ %d.\n",
				   pci_tbl[chip_idx].name, pciaddr, irq);

		if ((pci_tbl[chip_idx].flags & PCI_USES_IO)) {
			if (check_region(pciaddr, pci_tbl[chip_idx].io_size)
							&&check_region(pciaddr_dxx, pci_tbl[chip_idx].io_size_dxx)) /*Check for iospace wanted to be located for Cxx and Dxx */
				continue;
			ioaddr = pciaddr;
		
		} 
		else{
			if ((ioaddr = (long)ioremap(pciaddr & ~0xf, 0x200)) == 0)
			printk(KERN_INFO "Failed to map PCI address %#lx.\n",
				   pciaddr);
			else{
				
			}
			continue;
		}

		pcibios_read_config_word(pci_bus, pci_device_fn,
								 PCI_COMMAND, &pci_command);
		new_command = pci_command | (pci_tbl[chip_idx].flags & 7);
		if (pci_command != new_command) {
			printk(KERN_INFO "  The PCI BIOS has not enabled the"
				   " device at %d/%d!  Updating PCI command %4.4x->%4.4x.\n",
				   pci_bus, pci_device_fn, pci_command, new_command);
			pcibios_write_config_word(pci_bus, pci_device_fn,
									  PCI_COMMAND, new_command);
		}

		dev = pci_tbl[chip_idx].probe1(pci_bus, pci_device_fn, dev, ioaddr,
									   irq, chip_idx, cards_found);

		if (dev  && (pci_tbl[chip_idx].flags & PCI_COMMAND_MASTER)) {
			u8 pci_latency;
			pcibios_read_config_byte(pci_bus, pci_device_fn,
									 PCI_LATENCY_TIMER, &pci_latency);
			if (pci_latency < min_pci_latency) {
				printk(KERN_INFO "  PCI latency timer (CFLT) is "
					   "unreasonably low at %d.  Setting to %d clocks.\n",
					   pci_latency, min_pci_latency);
				pcibios_write_config_byte(pci_bus, pci_device_fn,
										  PCI_LATENCY_TIMER, min_pci_latency);
			}
		}
		dev = 0;
		cards_found++;
		printk(KERN_INFO "C_base=%lx,D_base=%lx",ioaddr,pciaddr_dxx);
	}

	return cards_found ? 0 : -ENODEV;
}

static struct device *w841_probe1(int pci_bus, int devfn, struct device *dev,
							   long ioaddr, int irq, int chip_id, int card_idx)
{
	struct netdev_private *np;
	int i, option = card_idx < MAX_UNITS ? options[card_idx] : 0;
	int j;

	dev = init_etherdev(dev, sizeof(struct netdev_private));

	printk(KERN_INFO "%s: %s at 0x%lx. ",
		   dev->name, skel_netdrv_tbl[chip_id].chip_name, ioaddr);

	/*Read hardware address from dxx StationAddr registers */
	for (j = 0; j < 6; j++)
		dev->dev_addr[j] = readb( pciaddr_dxx + StationAddr + j);

	for (i = 0; i < 6; i++)
		printk(KERN_INFO"%2.2x:", dev->dev_addr[i]);
			
	printk(KERN_INFO"%2.2x, IRQ %d.\n", dev->dev_addr[i], irq);

#ifdef USE_IO
	request_region(ioaddr, pci_tbl[chip_id].io_size, dev->name);
	request_region(pciaddr_dxx, pci_tbl[chip_id].io_size_dxx, dev->name);
#endif


#if ! defined(final_version) /* Dump the EEPROM contents during development. */
	if (debug > 4)
		for (i = 0; i < 64; i++)
			printk("%2.2x%s", eeprom_read(pciaddr_dxx, i), i%16 != 15 ? " " : "\n");
#endif

	/* Reset the chip to erase previous misconfiguration.
	   No hold time required! */
	writel(0x00000001, ioaddr + PCIBusCfg);

	dev->base_addr = ioaddr;
	dev->irq = irq;

	/* Make certain the descriptor lists are aligned. */
	np = (void *)(((long)kmalloc(sizeof(*np), GFP_KERNEL) + 15) & ~15);
	memset(np, 0, sizeof(*np));
	dev->priv = np;

	np->next_module = root_net_dev;
	root_net_dev = dev;

	np->pci_bus = pci_bus;
	np->pci_devfn = devfn;
	np->chip_id = chip_id;
	np->flags = skel_netdrv_tbl[np->chip_id].flags;

	if (dev->mem_start)
		option = dev->mem_start;

	/* The lower four bits are the media type. */
	if (option > 0) {
		if (option & 0x200)
			np->full_duplex = 1;
		np->default_port = option & 15;
		if (np->default_port)
			np->medialock = 1;
	}
	if (card_idx < MAX_UNITS  &&  full_duplex[card_idx] > 0)
		np->full_duplex = 1;

	if (np->full_duplex)
		np->duplex_lock = 1;

	/* The chip-specific entries in the device structure. */
	dev->open = &netdev_open;
	dev->hard_start_xmit = &start_tx;
	dev->stop = &netdev_close;
	dev->get_stats = &get_stats;
	dev->set_multicast_list = &set_rx_mode;
	dev->do_ioctl = &mii_ioctl;

	if (np->flags & CanHaveMII) {
		int phy, phy_idx = 0;
		for (phy = 1; phy < 32 && phy_idx < 4; phy++) {
			int mii_status = mdio_read(dev, phy, 1);
			if (mii_status != 0xffff  &&  mii_status != 0x0000) {
				np->phys[phy_idx++] = phy;
				np->advertising = mdio_read(dev, phy, 4);
				printk(KERN_INFO "%s: MII PHY found at address %d, status "
					   "0x%4.4x advertising %4.4x.\n",
					   dev->name, phy, mii_status, np->advertising);
			}
		}
		np->mii_cnt = phy_idx;
		if (phy_idx == 0) {
				printk(KERN_WARNING "%s: MII PHY not found -- this device may "
					   "not operate correctly.\n", dev->name);
		}
	}
        mdio_write(dev,np->phys[0],0,0x8000);
        do{
	;
	}while(0);
	

	return dev;
}



/* W841 eeprom_read function, access through the register Dc4 */
#define EE_SELECT		0x80000000
#define EE_START_READ	0x40000000
#define eeprom_delay()	do { ; } while (0)

static int eeprom_read(long baseaddr, int location)
{
	long ee_addr;
	int retval;
	long read_command =  EE_SELECT | ( location << 16 );

        ee_addr = baseaddr + EECtrl;
	writel( EE_SELECT , ee_addr );
    	eeprom_delay();
	writel( read_command , ee_addr ); 	
    	eeprom_delay();
    	writel( read_command | EE_START_READ , ee_addr );
	eeprom_delay();

	do{
	;
	}	
	while( readl(ee_addr) & EE_START_READ );

	retval= (int)( readl(ee_addr) & 0xffff );

	return retval;
	
}


/* W841 MII read and write functions*/
#define mdio_delay() readl(mdio_addr)
#define START_MDIO_RW 0x80000000
#define MDIO_READ_COMMAND 0x40000000
#define MDIO_WRITE_COMMAND 0x20000000

static int mdio_read(struct device *dev, int phy_id, int location)
{
	long mdio_addr = pciaddr_dxx + MIICtrl;
    	long start_mdio_read = START_MDIO_RW | MDIO_READ_COMMAND | ( phy_id << 21 ) | ( location << 16 );
	int retval;

	writel ( phy_id << 21 , mdio_addr );
	mdio_delay();
	writel ( ( phy_id << 21 ) | ( location << 16 ) , mdio_addr );
	mdio_delay();
	writel ( MDIO_READ_COMMAND | ( phy_id << 21 ) | ( location << 16 ) , mdio_addr );
	mdio_delay();
    	writel ( start_mdio_read , mdio_addr );


	do{
	;
	}	
	while( readl( mdio_addr ) & START_MDIO_RW );

	return retval= (int)( readl( mdio_addr ) & 0xffff );

}


static void mdio_write(struct device *dev, int phy_id, int location, int value)
{
	long mdio_addr = pciaddr_dxx + MIICtrl;
    	long start_mdio_write = START_MDIO_RW | MDIO_WRITE_COMMAND | ( phy_id << 21 ) | (location << 16) | value ;
	
	writel ( phy_id << 21 , mdio_addr );
	mdio_delay();
	writel ( ( phy_id << 21 ) | ( location << 16 ) , mdio_addr );
	mdio_delay();
	writel ( ( phy_id << 21 ) | ( location << 16 ) | value , mdio_addr );
	mdio_delay();
   	writel ( MDIO_WRITE_COMMAND | ( phy_id << 21 ) | ( location << 16 ) | value , mdio_addr );
	mdio_delay();
	writel ( start_mdio_write , mdio_addr );

	do{
	;
	}	
	while( readl( mdio_addr ) & START_MDIO_RW );
  
    return;

}


/*  */




static int netdev_open(struct device *dev)
{
	struct netdev_private *np = (struct netdev_private *)dev->priv;
	long ioaddr = dev->base_addr;
	int i;

	writel(0x00000001, ioaddr + PCIBusCfg);		/* Reset */

	if (request_irq(dev->irq, &intr_handler, SA_SHIRQ, dev->name, dev))
		return -EAGAIN;

	if (debug > 1)
		printk(KERN_DEBUG "%s: w89c841_open() irq %d.\n",
			   dev->name, dev->irq);

	MOD_INC_USE_COUNT;

	init_ring(dev);

	writel(virt_to_bus(np->rx_ring), ioaddr + RxRingPtr);
	writel(virt_to_bus(np->tx_ring), ioaddr + TxRingPtr);

	for (i = 0; i < 6; i++)
		writeb(dev->dev_addr[i], pciaddr_dxx + StationAddr + i);

	
	writel(0xE010, ioaddr + PCIBusCfg);


	if (dev->if_port == 0)
		dev->if_port = np->default_port;

	dev->tbusy = 0;
	dev->interrupt = 0;

	writel(0, dev->base_addr + RxStartDemand);
	np->csr6 = 0x20022004;	/*default value*/
	check_duplex(dev);
	set_rx_mode(dev);
       	
       	// printk(KERN_EMERG"%x\n",np->csr6); 
       
	dev->start = 1;

	/* Clear and Enable interrupts by setting the interrupt mask. */
	writel(0x1A0F5, ioaddr + IntrStatus);
	writel(0x1A0F5, ioaddr + IntrEnable);

	if (debug > 2)
		printk(KERN_DEBUG "%s: Done netdev_open().\n", dev->name);

	/* Set the timer to check for link beat. */
	init_timer(&np->timer);
	np->timer.expires = RUN_AT(3*HZ);
	np->timer.data = (unsigned long)dev;
	np->timer.function = &netdev_timer;				/* timer handler */
	add_timer(&np->timer);

	return 0;
}

static void check_duplex(struct device *dev)
{
	struct netdev_private *np = (struct netdev_private *)dev->priv;
	int mii_reg5 = mdio_read(dev, np->phys[0], 5);
	int duplex;

	if (np->duplex_lock  ||  mii_reg5 == 0xffff)
		return;
	duplex = (mii_reg5 & 0x0100) || (mii_reg5 & 0x01C0) == 0x0040;
	if (np->full_duplex != duplex) {
		np->full_duplex = duplex;
		if (debug)
			printk(KERN_INFO "%s: Setting %s-duplex based on MII #%d link"
				   " partner capability of %4.4x.\n", dev->name,
				   duplex ? "full" : "half", np->phys[0], mii_reg5);
		np->csr6 &= ~0x200;
		np->csr6 |= duplex ? 0x200 : 0;
	}
}

static void netdev_timer(unsigned long data)
{
	struct device *dev = (struct device *)data;
	struct netdev_private *np = (struct netdev_private *)dev->priv;
	long ioaddr = dev->base_addr;
	int next_tick = 10*HZ;
	int old_csr6 = np->csr6;

	if (debug > 2)
		printk(KERN_DEBUG "%s: Media selection timer tick, status %8.8x "
			   "config %8.8x.\n",
			   dev->name, readl(ioaddr + IntrStatus),
			   readl(ioaddr + NetworkConfig));
	check_duplex(dev);
	if (np->csr6 != old_csr6) {
		writel(np->csr6 & ~0x0004, ioaddr + NetworkConfig); //sam 0425 2002
		writel(np->csr6 | 0x2004, ioaddr + NetworkConfig); //sam 0425 2002 
	}
	np->timer.expires = RUN_AT(next_tick);
	add_timer(&np->timer);
}

static void tx_timeout(struct device *dev)
{
	struct netdev_private *np = (struct netdev_private *)dev->priv;
	long ioaddr = dev->base_addr;

	printk(KERN_WARNING "%s: Transmit timed out, status %8.8x,"
		   " resetting...\n", dev->name, readl(ioaddr + IntrStatus));

#ifndef __alpha__
	{
		int i;
		printk(KERN_DEBUG "  Rx ring %8.8x: ", (int)np->rx_ring);
		for (i = 0; i < RX_RING_SIZE; i++)
			printk(" %8.8x", (unsigned int)np->rx_ring[i].status);
		printk("\n"KERN_DEBUG"  Tx ring %8.8x: ", (int)np->tx_ring);
		for (i = 0; i < TX_RING_SIZE; i++)
			printk(" %4.4x", np->tx_ring[i].status);
		printk("\n");
	}
#endif

	/* Perhaps we should reinitialize the hardware here.  Just trigger a
	   Tx demand for now. */
	writel(0, dev->base_addr + TxStartDemand);
	dev->if_port = 0;
	/* Stop and restart the chip's Tx processes . */

	dev->trans_start = jiffies;
	np->stats.tx_errors++;
	return;
}


/* Initialize the Rx and Tx rings, along with various 'dev' bits. */
static void init_ring(struct device *dev)
{
	struct netdev_private *np = (struct netdev_private *)dev->priv;
	int i;

	np->tx_full = 0;
	np->cur_rx = np->cur_tx = 0;
	np->dirty_rx = np->dirty_tx = 0;

	np->rx_buf_sz = (dev->mtu <= 1500 ? PKT_BUF_SZ : dev->mtu + 32);
	np->rx_head_desc = &np->rx_ring[0];

	/* Initial all Rx descriptors. */
	for (i = 0; i < RX_RING_SIZE; i++) {
		np->rx_ring[i].length = np->rx_buf_sz;
		np->rx_ring[i].status = 0;
		np->rx_ring[i].next_desc = virt_to_bus(&np->rx_ring[i+1]);
		np->rx_skbuff[i] = 0;
	}
	/* Mark the last entry as wrapping the ring. */
	np->rx_ring[i-1].length |= DescEndRing;
	np->rx_ring[i-1].next_desc = virt_to_bus(&np->rx_ring[0]);

	/* Fill in the Rx buffers.  Handle allocation failure gracefully. */
	for (i = 0; i < RX_RING_SIZE; i++) {
		struct sk_buff *skb = dev_alloc_skb(np->rx_buf_sz);
		np->rx_skbuff[i] = skb;
		if (skb == NULL)
			break;
		skb->dev = dev;			/* Mark as being used by this device. */
		np->rx_ring[i].buffer1 = virt_to_bus(skb->tail);
		np->rx_ring[i].status = DescOwn | DescIntr;
	}
	np->dirty_rx = (unsigned int)(i - RX_RING_SIZE);

	for (i = 0; i < TX_RING_SIZE; i++) {
		np->tx_skbuff[i] = 0;
		np->tx_ring[i].status = 0;
	}
	return;
}

static int start_tx(struct sk_buff *skb, struct device *dev)
{
	struct netdev_private *np = (struct netdev_private *)dev->priv;
	unsigned entry;

	/* Block a timer-based transmit from overlapping.  This could better be
	   done with atomic_swap(1, dev->tbusy), but set_bit() works as well. */
	if (test_and_set_bit(0, (void*)&dev->tbusy) != 0) {
		if (jiffies - dev->trans_start > TX_TIMEOUT)
			tx_timeout(dev);
		return 1;
	}

	/* Caution: the write order is important here, set the field
	   with the "ownership" bits last. */

	/* Calculate the next Tx descriptor entry. */
	entry = np->cur_tx % TX_RING_SIZE;

	np->tx_skbuff[entry] = skb;
	np->tx_ring[entry].buffer1 = virt_to_bus(skb->data);

#define split_buffer
#define BPT 1022
#if defined(one_buffer)
	np->tx_ring[entry].length = DescWholePkt | skb->len;
	if (entry >= TX_RING_SIZE-1)		 /* Wrap ring */
		np->tx_ring[entry].length |= DescIntr | DescEndRing;
	np->tx_ring[entry].status = DescOwn;
	np->cur_tx++;
#elif defined(split_buffer)
	if (skb->len > BPT) {
		np->tx_ring[entry].buffer2 = virt_to_bus(skb->data + BPT);
		np->tx_ring[entry].length = DescWholePkt | ((skb->len - BPT)<<11) | BPT;
	} else
		np->tx_ring[entry].length = DescWholePkt | skb->len;
	if (entry >= TX_RING_SIZE-1)		 /* Wrap ring */
		np->tx_ring[entry].length |= DescIntr | DescEndRing;
	np->tx_ring[entry].status = DescOwn;
	np->cur_tx++;
#elif defined(two_buffer)
	if (skb->len > BPT) {
		unsigned int entry1 = ++np->cur_tx % TX_RING_SIZE;
		np->tx_ring[entry].length = DescStartPkt | BPT;
		np->tx_ring[entry1].length = DescEndPkt | (skb->len - BPT);
		np->tx_ring[entry1].buffer1 = virt_to_bus(skb->data) + BPT;
		np->tx_ring[entry1].status = DescOwn;
		np->tx_ring[entry].status = DescOwn;
		if (entry >= TX_RING_SIZE-1)
			np->tx_ring[entry].length |= DescIntr|DescEndRing;
		else if (entry1 >= TX_RING_SIZE-1)
			np->tx_ring[entry1].length |= DescIntr|DescEndRing;
		np->cur_tx++;
	} else {
		np->tx_ring[entry].length = DescWholePkt | skb->len;
		if (entry >= TX_RING_SIZE-1)		 /* Wrap ring */
			np->tx_ring[entry].length |= DescIntr | DescEndRing;
		np->tx_ring[entry].status = DescOwn;
		np->cur_tx++;
	}
#endif

	writel(0, dev->base_addr + TxStartDemand);
	if (np->cur_tx - np->dirty_tx > TX_RING_SIZE - 2)
		np->tx_full = 1;
	/* Work around horrible bug in the chip! */
	else if (skb->len > 1028  &&  (np->flags & HasBrokenTx))
		np->tx_full = 1;
	else 
		clear_bit(0, (void*)&dev->tbusy);		/* Typical path */

	dev->trans_start = jiffies;

	if (debug > 4) {
		printk(KERN_DEBUG "%s: Transmit frame #%d queued in slot %d.\n",
			   dev->name, np->cur_tx, entry);
	}
	return 0;
}

/* The interrupt handler does all of the Rx thread work and cleans up
   after the Tx thread. */
static void intr_handler(int irq, void *dev_instance, struct pt_regs *rgs)
{
	struct device *dev = (struct device *)dev_instance;
	struct netdev_private *np;
	long ioaddr, boguscnt = max_interrupt_work;

#ifndef final_version			/* Can never occur. */
	if (dev == NULL) {
		printk (KERN_ERR "Netdev interrupt handler(): IRQ %d for unknown "
				"device.\n", irq);
		return;
	}
#endif

	ioaddr = dev->base_addr;
	np = (struct netdev_private *)dev->priv;
#if defined(__i386__)
	/* A lock to prevent simultaneous entry bug on Intel SMP machines. */
	if (test_and_set_bit(0, (void*)&dev->interrupt)) {
		printk(KERN_ERR"%s: SMP simultaneous entry of an interrupt handler.\n",
			   dev->name);
		dev->interrupt = 0;	/* Avoid halting machine. */
		return;
	}
#else
	if (dev->interrupt) {
		printk(KERN_ERR "%s: Re-entering the interrupt handler.\n", dev->name);
		return;
	}
	dev->interrupt = 1;
#endif

	do {
		u32 intr_status = readl(ioaddr + IntrStatus);

		/* Acknowledge all of the current interrupt sources ASAP. */
		writel(intr_status & 0x001ffff, ioaddr + IntrStatus);

		if (debug > 4)
			printk(KERN_DEBUG "%s: Interrupt, status %4.4x.\n",
				   dev->name, intr_status);

		if ((intr_status & (NormalIntr|AbnormalIntr)) == 0)
			break;

		if (intr_status & (IntrRxDone | RxNoBuf))
			netdev_rx(dev);

		for (; np->cur_tx - np->dirty_tx > 0; np->dirty_tx++) {
			int entry = np->dirty_tx % TX_RING_SIZE;
			int tx_status = np->tx_ring[entry].status;

			if (tx_status < 0)
				break;
			if (tx_status & 0x8000) { 		/* There was an error, log it. */
#ifndef final_version
				if (debug > 1)
					printk(KERN_DEBUG "%s: Transmit error, Tx status %8.8x.\n",
						   dev->name, tx_status);
#endif
				np->stats.tx_errors++;
				if (tx_status & 0x0104) np->stats.tx_aborted_errors++;
				if (tx_status & 0x0C80) np->stats.tx_carrier_errors++;
				if (tx_status & 0x0200) np->stats.tx_window_errors++;
				if (tx_status & 0x0002) np->stats.tx_fifo_errors++;
				if ((tx_status & 0x0080) && np->full_duplex == 0)
					np->stats.tx_heartbeat_errors++;
#ifdef ETHER_STATS
				if (tx_status & 0x0100) np->stats.collisions16++;
#endif
			} else {
#ifdef ETHER_STATS
				if (tx_status & 0x0001) np->stats.tx_deferred++;
#endif
#if LINUX_VERSION_CODE > 0x20127
				np->stats.tx_bytes += np->tx_ring[entry].length & 0x7ff;
#endif
				np->stats.collisions += (tx_status >> 3) & 15;
				np->stats.tx_packets++;
			}
			/* Free the original skb. */
			dev_free_skb(np->tx_skbuff[entry]);
			np->tx_skbuff[entry] = 0;
		}
		if (np->tx_full && np->cur_tx - np->dirty_tx < TX_RING_SIZE - 4) {
			/* The ring is no longer full, clear tbusy. */
			np->tx_full = 0;
			clear_bit(0, (void*)&dev->tbusy);
			mark_bh(NET_BH);
		}

		/* Abnormal error summary/uncommon events handlers. */
		if (intr_status & (AbnormalIntr | TxFIFOUnderflow | IntrPCIErr |
						   TimerInt | IntrTxStopped))
			netdev_error(dev, intr_status);

		if (--boguscnt < 0) {
			printk(KERN_WARNING "%s: Too much work at interrupt, "
				   "status=0x%4.4x.\n",
				   dev->name, intr_status);
			break;
		}
	} while (1);

	if (debug > 3)
		printk(KERN_DEBUG "%s: exiting interrupt, status=%#4.4x.\n",
			   dev->name, readl(ioaddr + IntrStatus));

#if defined(__i386__)
	clear_bit(0, (void*)&dev->interrupt);
#else
	dev->interrupt = 0;
#endif
	return;
}

/* This routine is logically part of the interrupt handler, but seperated
   for clarity and better register allocation. */
static int netdev_rx(struct device *dev)
{
	struct netdev_private *np = (struct netdev_private *)dev->priv;
	int entry = np->cur_rx % RX_RING_SIZE;
	int boguscnt = np->dirty_rx + RX_RING_SIZE - np->cur_rx;

	if (debug > 4) {
		printk(KERN_DEBUG " In netdev_rx(), entry %d status %4.4x.\n",
			   entry, np->rx_ring[entry].status);
	}

	/* If EOP is set on the next entry, it's a new packet. Send it up. */
	while (np->rx_head_desc->status >= 0) {
		struct w841_rx_desc *desc = np->rx_head_desc;
		s32 status = desc->status;

		if (debug > 4)
			printk(KERN_DEBUG "  netdev_rx() status was %8.8x.\n",
				   status);
		if (--boguscnt < 0)
			break;
		if ((status & 0x38008300) != 0x0300) {
			if ((status & 0x38000300) != 0x0300) {
				/* Ingore earlier buffers. */
				if ((status & 0xffff) != 0x7fff) {
					printk(KERN_WARNING "%s: Oversized Ethernet frame spanned "
						   "multiple buffers, entry %#x status %4.4x!\n",
						   dev->name, np->cur_rx, status);
					np->stats.rx_length_errors++;
				}
			} else if (status & 0x8000) {
				/* There was a fatal error. */
				if (debug > 2)
					printk(KERN_DEBUG "%s: Receive error, Rx status %8.8x.\n",
						   dev->name, status);
				np->stats.rx_errors++; /* end of a packet.*/
				if (status & 0x0890) np->stats.rx_length_errors++;
				if (status & 0x004C) np->stats.rx_frame_errors++;
				if (status & 0x0002) np->stats.rx_crc_errors++;
			}
		} else {
			struct sk_buff *skb;
			/* Omit the four octet CRC from the length. */
			short pkt_len = ((status >> 16) & 0x7ff) - 4;

#ifndef final_version
			if (debug > 4)
				printk(KERN_DEBUG "  netdev_rx() normal Rx pkt length %d"
					   " status %x.\n", pkt_len, status);
#endif
			/* Check if the packet is long enough to accept without copying
			   to a minimally-sized skbuff. */
			if (pkt_len < rx_copybreak
				&& (skb = dev_alloc_skb(pkt_len + 2)) != NULL) {
				skb->dev = dev;
				skb_reserve(skb, 2);	/* 16 byte align the IP header */
				/* Call copy + cksum if available. */
#if ! defined(__alpha__)
				eth_copy_and_sum(skb, bus_to_virt(desc->buffer1),
								 pkt_len, 0);
				skb_put(skb, pkt_len);
#else
				memcpy(skb_put(skb, pkt_len),
					   bus_to_virt(desc->buffer1), pkt_len);
#endif
			} else {
				char *temp = skb_put(skb = np->rx_skbuff[entry], pkt_len);
				np->rx_skbuff[entry] = NULL;
#ifndef final_version				/* Remove after testing. */
				if (bus_to_virt(desc->buffer1) != temp)
					printk(KERN_ERR "%s: Internal fault: The skbuff addresses "
						   "do not match in netdev_rx: %p vs. %p / %p.\n",
						   dev->name, bus_to_virt(desc->buffer1),
						   skb->head, temp);
#endif
			}
#ifndef final_version				/* Remove after testing. */
			/* You will want this info for the initial debug. */
			if (debug > 5)
				printk(KERN_DEBUG "  Rx data %2.2x:%2.2x:%2.2x:%2.2x:%2.2x:"
					   "%2.2x %2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x %2.2x%2.2x "
					   "%d.%d.%d.%d.\n",
					   skb->data[0], skb->data[1], skb->data[2], skb->data[3],
					   skb->data[4], skb->data[5], skb->data[6], skb->data[7],
					   skb->data[8], skb->data[9], skb->data[10],
					   skb->data[11], skb->data[12], skb->data[13],
					   skb->data[14], skb->data[15], skb->data[16],
					   skb->data[17]);
#endif
			skb->protocol = eth_type_trans(skb, dev);
			netif_rx(skb);
			dev->last_rx = jiffies;
			np->stats.rx_packets++;
#if LINUX_VERSION_CODE > 0x20127
			np->stats.rx_bytes += pkt_len;
#endif
		}
		entry = (++np->cur_rx) % RX_RING_SIZE;
		np->rx_head_desc = &np->rx_ring[entry];
	}

	/* Refill the Rx ring buffers. */
	for (; np->cur_rx - np->dirty_rx > 0; np->dirty_rx++) {
		struct sk_buff *skb;
		entry = np->dirty_rx % RX_RING_SIZE;
		if (np->rx_skbuff[entry] == NULL) {
			skb = dev_alloc_skb(np->rx_buf_sz);
			np->rx_skbuff[entry] = skb;
			if (skb == NULL)
				break;			/* Better luck next round. */
			skb->dev = dev;			/* Mark as being used by this device. */
			np->rx_ring[entry].buffer1 = virt_to_bus(skb->tail);
		}
		np->rx_ring[entry].status = DescOwn;
	}

	return 0;
}

static void netdev_error(struct device *dev, int intr_status)
{
	long ioaddr = dev->base_addr;
	struct netdev_private *np = (struct netdev_private *)dev->priv;

	if (debug > 2)
		printk(KERN_DEBUG "%s: Abnormal event, %8.8x.\n",
			   dev->name, intr_status);
	if (intr_status == 0xffffffff)
		return;
	if (intr_status & TxFIFOUnderflow) {
		np->csr6 += 0x4000;	/* Bump up the Tx threshold */
		printk(KERN_DEBUG "%s: Tx underflow, increasing threshold to %8.8x.\n",
			   dev->name, np->csr6);
		writel(np->csr6, ioaddr + NetworkConfig);
	}
	if (intr_status & IntrRxDied) {		/* Missed a Rx frame. */
		np->stats.rx_errors++;
	}
	if (intr_status & TimerInt) {
		printk(KERN_ERR "%s: Unexpected timer interrupt! %8.8x.\n",
			   dev->name, intr_status);
		/* Hmmmmm, it's not clear what to do here. */
	}
	np->stats.rx_missed_errors += readl(ioaddr + RxMissed) & 0xffff;
	writel(0, dev->base_addr + RxStartDemand);
}

static struct enet_statistics *get_stats(struct device *dev)
{
	long ioaddr = dev->base_addr;
	struct netdev_private *np = (struct netdev_private *)dev->priv;

	/* The chip only need report frame silently dropped. */
	if (dev->start)
		np->stats.rx_missed_errors += readl(ioaddr + RxMissed) & 0x7fff;

	return &np->stats;
}

static unsigned const ethernet_polynomial = 0x04c11db7U;
static inline u32 ether_crc(int length, unsigned char *data)
{
    int crc = -1;

    while(--length >= 0) {
		unsigned char current_octet = *data++;
		int bit;
		for (bit = 0; bit < 8; bit++, current_octet >>= 1) {
			crc = (crc << 1) ^
				((crc < 0) ^ (current_octet & 1) ? ethernet_polynomial : 0);
		}
    }
    return crc;
}

static void set_rx_mode(struct device *dev)
{
	struct netdev_private *np = (struct netdev_private *)dev->priv;
	long ioaddr = dev->base_addr;
	u32 mc_filter[2];			/* Multicast hash filter */
	u32 rx_mode;

	if (dev->flags & IFF_PROMISC) {			/* Set promiscuous. */
		/* Unconditionally log net taps. */
		printk(KERN_NOTICE "%s: Promiscuous mode enabled.\n", dev->name);
		memset(mc_filter, 0xff, sizeof(mc_filter));
		rx_mode = AcceptBroadcast | AcceptMulticast | AcceptAllPhys
			| AcceptMyPhys;
	} else if ((dev->mc_count > multicast_filter_limit)
			   ||  (dev->flags & IFF_ALLMULTI)) {
		/* Too many to match, or accept all multicasts. */
		memset(mc_filter, 0xff, sizeof(mc_filter));
		rx_mode = AcceptBroadcast | AcceptMulticast | AcceptMyPhys| AcceptAllPhys;
	} else {
		struct dev_mc_list *mclist;
		int i;
		memset(mc_filter, 0, sizeof(mc_filter));
		for (i = 0, mclist = dev->mc_list; mclist && i < dev->mc_count;
			 i++, mclist = mclist->next) {
			set_bit((ether_crc(ETH_ALEN, mclist->dmi_addr) >> 26) ^ 0x3F,
					mc_filter);
		}
		rx_mode = AcceptBroadcast | AcceptMulticast | AcceptMyPhys| AcceptAllPhys;
	}
	writel(mc_filter[0], ioaddr + MulticastFilter0);
	writel(mc_filter[1], ioaddr + MulticastFilter1);
	np->csr6 &= ~0x00F8;
	np->csr6 |= rx_mode;
	np->csr6 |=AcceptAllPhys;
	writel(np->csr6, ioaddr + NetworkConfig);
}

static int mii_ioctl(struct device *dev, struct ifreq *rq, int cmd)
{
	u16 *data = (u16 *)&rq->ifr_data;

	switch(cmd) {
	case SIOCDEVPRIVATE:		/* Get the address of the PHY in use. */
		data[0] = ((struct netdev_private *)dev->priv)->phys[0] & 0x1f;
		/* Fall Through */
	case SIOCDEVPRIVATE+1:		/* Read the specified MII register. */
		data[3] = mdio_read(dev, data[0] & 0x1f, data[1] & 0x1f);
		return 0;
	case SIOCDEVPRIVATE+2:		/* Write the specified MII register */
		if (!suser())
			return -EPERM;
		mdio_write(dev, data[0] & 0x1f, data[1] & 0x1f, data[2]);
		return 0;
	default:
		return -EOPNOTSUPP;
	}
}

static int netdev_close(struct device *dev)
{
	long ioaddr = dev->base_addr;
	struct netdev_private *np = (struct netdev_private *)dev->priv;
	int i;

	dev->start = 0;
	dev->tbusy = 1;

	if (debug > 1) {
		printk(KERN_DEBUG "%s: Shutting down ethercard, status was %8.8x "
			   "Config %8.8x.\n", dev->name, readl(ioaddr + IntrStatus),
			   readl(ioaddr + NetworkConfig));
		printk(KERN_DEBUG "%s: Queue pointers were Tx %d / %d,  Rx %d / %d.\n",
			   dev->name, np->cur_tx, np->dirty_tx, np->cur_rx, np->dirty_rx);
	}

	/* Disable interrupts by clearing the interrupt mask. */
	writel(0x0000, ioaddr + IntrEnable);

	/* Stop the chip's Tx and Rx processes. */
	writel(np->csr6 &= ~0x20FA, ioaddr + NetworkConfig);

	del_timer(&np->timer);
	if (readl(ioaddr + NetworkConfig) != 0xffffffff)
		np->stats.rx_missed_errors += readl(ioaddr + RxMissed) & 0xffff;

#ifdef __i386__
	if (debug > 2) {
		printk("\n"KERN_DEBUG"  Tx ring at %8.8x:\n",
			   (int)virt_to_bus(np->tx_ring));
		for (i = 0; i < TX_RING_SIZE; i++)
			printk(" #%d desc. %4.4x %4.4x %8.8x.\n",
				   i, np->tx_ring[i].length,
				   np->tx_ring[i].status, np->tx_ring[i].buffer1);
		printk("\n"KERN_DEBUG "  Rx ring %8.8x:\n",
			   (int)virt_to_bus(np->rx_ring));
		for (i = 0; i < RX_RING_SIZE; i++) {
			printk(KERN_DEBUG " #%d desc. %4.4x %4.4x %8.8x\n",
				   i, np->rx_ring[i].length,
				   np->rx_ring[i].status, np->rx_ring[i].buffer1);
		}
	}
#endif /* __i386__ debugging only */

	free_irq(dev->irq, dev);

	/* Free all the skbuffs in the Rx queue. */
	for (i = 0; i < RX_RING_SIZE; i++) {
		np->rx_ring[i].status = 0;
		if (np->rx_skbuff[i]) {
#if LINUX_VERSION_CODE < 0x20100
			np->rx_skbuff[i]->free = 1;
#endif
			dev_free_skb(np->rx_skbuff[i]);
		}
		np->rx_skbuff[i] = 0;
	}
	for (i = 0; i < TX_RING_SIZE; i++) {
		if (np->tx_skbuff[i])
			dev_free_skb(np->tx_skbuff[i]);
		np->tx_skbuff[i] = 0;
	}

	MOD_DEC_USE_COUNT;

	return 0;
}


#ifdef MODULE
int init_module(void)
{
	if (debug)					/* Emit version even if no cards detected. */
		printk(KERN_INFO "%s", version);
#ifdef CARDBUS
	register_driver(&etherdev_ops);
	return 0;
#else
	return pci_etherdev_probe(NULL, pci_tbl);
#endif
}

void cleanup_module(void)
{
	struct device *next_dev;

#ifdef CARDBUS
	unregister_driver(&etherdev_ops);
#endif

	/* No need to check MOD_IN_USE, as sys_delete_module() checks. */
	while (root_net_dev) {
		next_dev = ((struct netdev_private *)root_net_dev->priv)->next_module;
		unregister_netdev(root_net_dev);
#ifdef USE_IO
		release_region(root_net_dev->base_addr, pci_tbl[0].io_size);
		release_region(pciaddr_dxx, pci_tbl[0].io_size_dxx);
#else
		iounmap((char *)(root_net_dev->base_addr));
#endif
		kfree(root_net_dev);
		root_net_dev = next_dev;
	}
}
#else
int winbond841_probe(struct device *dev)
{
	printk(KERN_INFO "%s", version);
	return pci_etherdev_probe(dev, pci_tbl);
}
#endif  /* MODULE */


/*
 * Local variables:
 *  compile-command: "gcc -DMODULE -D__KERNEL__ -I/usr/src/linux/net/inet -Wall -Wstrict-prototypes -O6 -c winbond841.c `[ -f /usr/include/linux/modversions.h ] && echo -DMODVERSIONS`"
 *  SMP-compile-command: "gcc -D__SMP__ -DMODULE -D__KERNEL__ -I/usr/src/linux/net/inet -Wall -Wstrict-prototypes -O6 -c winbond-841.c `[ -f /usr/include/linux/modversions.h ] && echo -DMODVERSIONS`"
 *  c-indent-level: 4
 *  c-basic-offset: 4
 *  tab-width: 4
 * End:
 */
